@@ -13,7 +13,6 @@ use rash_derive::FieldNames;
 use std::fs;
 use std::path::PathBuf;
 
-use serde_json::Value;
 use yaml_rust::{Yaml, YamlLoader};
 
 /// Main structure at definition level which prepares [`Module`] executions.
@@ -25,6 +24,8 @@ use yaml_rust::{Yaml, YamlLoader};
 #[derive(Debug, Clone, PartialEq, FieldNames)]
 // ANCHOR: task
 pub struct Task {
+    /// Run task in dry-run mode without modifications.
+    check_mode: bool,
     /// Module could be any [`Module`] accessible by its name.
     ///
     /// [`Module`]: ../modules/struct.Module.html
@@ -33,18 +34,22 @@ pub struct Task {
     ///
     /// [`Module::exec`]: ../modules/struct.Module.html#method.exec
     params: Yaml,
+    /// Overwrite changed field in [`ModuleResult`].
+    ///
+    /// [`ModuleResult`]: ../modules/struct.ModuleResult.html
+    changed_when: Option<String>,
+    /// Template expression passed directly without {{ }}; if true errors are ignored.
+    ignore_errors: Option<bool>,
     /// Task name.
     name: Option<String>,
-    /// Template expression passed directly without {{ }}; if false skip task execution.
-    when: Option<String>,
-    /// Variable name to store [`ModuleOutput`].
+    /// `loop` field receives a Template (with {{ }}) or a list to iterate over it.
+    r#loop: Option<Yaml>,
+    /// Variable name to store [`ModuleResult`].
     ///
     /// [`ModuleResult`]: ../modules/struct.ModuleResult.html
     register: Option<String>,
-    /// Template expression passed directly without {{ }}; if true errors are ignored.
-    ignore_errors: Option<bool>,
-    /// `loop` field receives a Template (with {{ }}) or a list to iterate over it.
-    r#loop: Option<Yaml>,
+    /// Template expression passed directly without {{ }}; if false skip task execution.
+    when: Option<String>,
 }
 // ANCHOR_END: task
 
@@ -62,9 +67,9 @@ impl Task {
     ///
     /// [`Task`]: struct.Task.html
     /// [`Yaml`]: ../../yaml_rust/struct.Yaml.html
-    pub fn new(yaml: &Yaml) -> Result<Self> {
+    pub fn new(yaml: &Yaml, check: bool) -> Result<Self> {
         trace!("new task: {:?}", yaml);
-        TaskNew::from(yaml).validate_attrs()?.get_task()
+        TaskNew::from(yaml).validate_attrs()?.get_task(check)
     }
 
     #[inline(always)]
@@ -79,7 +84,7 @@ impl Task {
                 .clone()
                 .iter()
                 .map(|t| match &t.1.clone().as_str() {
-                    Some(s) => match render_string(s, vars.clone()) {
+                    Some(s) => match render_string(s, &vars) {
                         Ok(s) => Ok((t.0.clone(), Yaml::String(s))),
                         Err(e) => Err(e),
                     },
@@ -94,7 +99,7 @@ impl Task {
                                 )),
                             })
                             .map(|result_s| match result_s {
-                                Ok(s) => match render_string(&s, vars.clone()) {
+                                Ok(s) => match render_string(&s, &vars) {
                                     Ok(rendered_s) => Ok(Yaml::String(rendered_s)),
                                     Err(e) => Err(e),
                                 },
@@ -121,12 +126,13 @@ impl Task {
                         format!("{:?} must be a string", original_params),
                     )
                 })?,
-                vars,
+                &vars,
             )?)),
         }
     }
 
-    fn is_exec(&self, vars: Vars) -> Result<bool> {
+    fn is_exec(&self, vars: &Vars) -> Result<bool> {
+        trace!("when: {:?}", &self.when);
         match &self.when {
             Some(s) => is_render_string(s, vars),
             None => Ok(true),
@@ -138,8 +144,8 @@ impl Task {
             Some(v) => Ok(v
                 .iter()
                 .map(|item| match item.clone() {
-                    Yaml::Real(s) | Yaml::String(s) => Ok(render_string(&s, vars.clone())?),
-                    Yaml::Integer(x) => Ok(render_string(&x.to_string(), vars.clone())?),
+                    Yaml::Real(s) | Yaml::String(s) => Ok(render_string(&s, &vars)?),
+                    Yaml::Integer(x) => Ok(render_string(&x.to_string(), &vars)?),
                     _ => Err(Error::new(
                         ErrorKind::InvalidData,
                         format!("{:?} is not a valid string", item),
@@ -155,7 +161,7 @@ impl Task {
         let loop_some = self.r#loop.clone().unwrap();
         match loop_some.as_str() {
             Some(s) => {
-                let yaml = get_yaml(&render_as_json(s, vars.clone())?)?;
+                let yaml = get_yaml(&render_as_json(s, &vars)?)?;
                 match yaml.as_str() {
                     Some(s) => Ok(vec![s.to_string()]),
                     None => Task::get_iterator(&yaml, vars),
@@ -165,6 +171,54 @@ impl Task {
         }
     }
 
+    fn is_changed(&self, result: &ModuleResult, vars: &Vars) -> Result<bool> {
+        trace!("changed_when: {:?}", &self.changed_when);
+        match &self.changed_when {
+            Some(s) => is_render_string(s, vars),
+            None => Ok(result.get_changed()),
+        }
+    }
+
+    fn exec_module(&self, vars: Vars) -> Result<Vars> {
+        if self.is_exec(&vars)? {
+            let rendered_params = self.render_params(vars.clone())?;
+            match self
+                .module
+                .exec(rendered_params.clone(), vars.clone(), self.check_mode)
+            {
+                Ok((result, result_vars)) => {
+                    info!(target: if self.is_changed(&result, &result_vars)? {"changed"} else { "ok"},
+                        "{}",
+                        result.get_output().unwrap_or_else(
+                            || format!("{:?}", rendered_params)
+                        )
+                    );
+                    let mut new_vars = result_vars;
+                    if self.register.is_some() {
+                        let register = self.register.as_ref().unwrap();
+                        trace!("register {:?} in {:?}", &result, register);
+                        new_vars.insert(register, &result);
+                    }
+                    Ok(new_vars)
+                }
+                Err(e) => match self.ignore_errors {
+                    Some(is_true) => {
+                        if is_true {
+                            info!(target: "ignoring", "{}", e);
+                            Ok(vars)
+                        } else {
+                            Err(e)
+                        }
+                    }
+                    None => Err(e),
+                },
+            }
+        } else {
+            info!(target: "skipping", "");
+            Ok(vars)
+        }
+    }
+
     /// Execute [`Module`] rendering `self.params` with [`Vars`].
     ///
     /// [`Module`]: ../modules/struct.Module.html
@@ -173,91 +227,17 @@ impl Task {
         debug!("Module: {}", self.module.get_name());
         debug!("Params: {:?}", self.params);
 
-        let new_vars = if self.is_exec(vars.clone())? {
-            let result_json_vars: Result<(Value, Vars)> = if self.r#loop.is_some() {
-                let results_with_vars = self
-                    .render_iterator(vars.clone())?
-                    .into_iter()
-                    .map(|item| {
-                        let mut exec_vars = vars.clone();
-                        exec_vars.insert("item", &item);
-                        let rendered_params = self.render_params(exec_vars.clone())?;
-                        let result_wrapped =
-                            self.module.exec(rendered_params.clone(), exec_vars.clone());
-                        match result_wrapped {
-                            Ok((result, new_vars)) => {
-                                info!(target: if result.get_changed() {"changed"} else { "ok"},
-                                    "{:?}",
-                                    result.get_output().unwrap_or_else(
-                                        || format!("{:?}", rendered_params)
-                                    )
-                                );
-                                Ok((result, new_vars))
-                            }
-                            Err(e) => match self.ignore_errors {
-                                Some(is_true) => {
-                                    if is_true {
-                                        info!(target: "ignoring", "{}", e);
-                                        Ok((ModuleResult::new(false, None, None), exec_vars))
-                                    } else {
-                                        Err(e)
-                                    }
-                                }
-                                None => Err(e),
-                            },
-                        }
-                    })
-                    .collect::<Result<Vec<(ModuleResult, Vars)>>>()?;
-                let mut new_vars = Vars::new();
-                results_with_vars
-                    .iter()
-                    .for_each(|(_, vars)| new_vars.extend(vars.clone()));
-                let results: Vec<ModuleResult> = results_with_vars
-                    .iter()
-                    .map(|(result, _)| result)
-                    .cloned()
-                    .collect();
-                Ok((json!(results), new_vars))
-            } else {
-                let rendered_params = self.render_params(vars.clone())?;
-                let (result, new_vars) =
-                    match self.module.exec(rendered_params.clone(), vars.clone()) {
-                        Ok((result, new_vars)) => {
-                            info!(target: if result.get_changed() {"changed"} else { "ok"},
-                                "{}",
-                                result.get_output().unwrap_or_else(
-                                    || format!("{:?}", rendered_params)
-                                )
-                            );
-                            Ok((result, new_vars))
-                        }
-                        Err(e) => match self.ignore_errors {
-                            Some(is_true) => {
-                                if is_true {
-                                    info!(target: "ignoring", "{}", e);
-                                    Ok((ModuleResult::new(false, None, None), vars))
-                                } else {
-                                    Err(e)
-                                }
-                            }
-                            None => Err(e),
-                        },
-                    }?;
-                Ok((json!(result), new_vars))
-            };
-            let json_vars = result_json_vars?;
-            let result = json_vars.0;
-            let mut new_vars = json_vars.1;
-            if self.register.is_some() {
-                new_vars.insert(self.register.as_ref().unwrap(), &result);
+        if self.r#loop.is_some() {
+            let mut new_vars = vars.clone();
+            for item in self.render_iterator(vars)?.into_iter() {
+                new_vars.insert("item", &item);
+                new_vars = self.exec_module(new_vars.clone())?;
             }
-            new_vars
+            Ok(new_vars)
         } else {
-            info!(target: "skipping", "");
-            vars
-        };
-
-        Ok(new_vars)
+            let new_vars = self.exec_module(vars)?;
+            Ok(new_vars)
+        }
     }
 
     /// Return name.
@@ -274,7 +254,7 @@ impl Task {
                 .name
                 .clone()
                 .ok_or_else(|| Error::new(ErrorKind::NotFound, "no name found"))?,
-            vars,
+            &vars,
         )
     }
 
@@ -288,22 +268,24 @@ impl Task {
     #[cfg(test)]
     pub fn test_example() -> Self {
         Task {
+            check_mode: false,
             module: Module::test_example(),
-            name: None,
-            when: None,
-            register: None,
-            ignore_errors: None,
-            r#loop: None,
             params: YamlLoader::load_from_str("cmd: ls")
                 .unwrap()
                 .first()
                 .unwrap()
                 .clone(),
+            changed_when: None,
+            ignore_errors: None,
+            name: None,
+            r#loop: None,
+            register: None,
+            when: None,
         }
     }
 }
 
-pub fn read_file(tasks_file_path: PathBuf) -> Result<Tasks> {
+pub fn read_file(tasks_file_path: PathBuf, check: bool) -> Result<Tasks> {
     trace!("reading tasks from: {:?}", tasks_file_path);
     let tasks_file =
         fs::read_to_string(tasks_file_path).map_err(|e| Error::new(ErrorKind::InvalidData, e))?;
@@ -318,7 +300,7 @@ pub fn read_file(tasks_file_path: PathBuf) -> Result<Tasks> {
 
     yaml.clone()
         .into_iter()
-        .map(|task| Task::new(&task))
+        .map(|task| Task::new(&task, check))
         .collect::<Result<Tasks>>()
 }
 
@@ -328,7 +310,7 @@ impl From<&Yaml> for Task {
         TaskNew::from(yaml)
             .validate_attrs()
             .unwrap()
-            .get_task()
+            .get_task(false)
             .unwrap()
     }
 }
@@ -345,6 +327,7 @@ mod tests {
     use std::io::Write;
 
     use tempfile::tempdir;
+    use tera::Context;
     use yaml_rust::YamlLoader;
 
     #[test]
@@ -371,7 +354,7 @@ mod tests {
         .to_owned();
         let out = YamlLoader::load_from_str(&s).unwrap();
         let yaml = out.first().unwrap();
-        let task_err = Task::new(yaml).unwrap_err();
+        let task_err = Task::new(yaml, false).unwrap_err();
         assert_eq!(task_err.kind(), ErrorKind::InvalidData);
     }
 
@@ -385,7 +368,7 @@ mod tests {
         .to_owned();
         let out = YamlLoader::load_from_str(&s).unwrap();
         let yaml = out.first().unwrap();
-        let task_err = Task::new(yaml).unwrap_err();
+        let task_err = Task::new(yaml, false).unwrap_err();
         assert_eq!(task_err.kind(), ErrorKind::InvalidData);
     }
 
@@ -400,11 +383,11 @@ mod tests {
         let out = YamlLoader::load_from_str(&s).unwrap();
         let yaml = out.first().unwrap();
         let task = Task::from(yaml);
-        assert_eq!(task.is_exec(vars).unwrap(), true);
+        assert_eq!(task.is_exec(&vars).unwrap(), true);
     }
 
     #[test]
-    fn test_is_exec_bool() {
+    fn test_is_exec_parsed_bool() {
         let s: String = r#"
         when: "boo | bool"
         command: 'example'
@@ -414,7 +397,7 @@ mod tests {
         let out = YamlLoader::load_from_str(&s).unwrap();
         let yaml = out.first().unwrap();
         let task = Task::from(yaml);
-        assert_eq!(task.is_exec(vars).unwrap(), false);
+        assert_eq!(task.is_exec(&vars).unwrap(), false);
     }
 
     #[test]
@@ -428,7 +411,53 @@ mod tests {
         let out = YamlLoader::load_from_str(&s).unwrap();
         let yaml = out.first().unwrap();
         let task = Task::from(yaml);
-        assert_eq!(task.is_exec(vars).unwrap(), false);
+        assert_eq!(task.is_exec(&vars).unwrap(), false);
+    }
+
+    #[test]
+    fn test_is_exec_bool_false() {
+        let s: String = r#"
+        when: false
+        command: 'example'
+        "#
+        .to_owned();
+        let vars = vars::from_iter(vec![].into_iter());
+        let out = YamlLoader::load_from_str(&s).unwrap();
+        let yaml = out.first().unwrap();
+        let task = Task::from(yaml);
+        assert_eq!(task.is_exec(&vars).unwrap(), false);
+    }
+
+    #[test]
+    fn test_is_exec_array() {
+        let s: String = r#"
+        when:
+          - true
+          - "boo == 'test'"
+        command: 'example'
+        "#
+        .to_owned();
+        let vars = vars::from_iter(vec![("boo", "test")].into_iter());
+        let out = YamlLoader::load_from_str(&s).unwrap();
+        let yaml = out.first().unwrap();
+        let task = Task::from(yaml);
+        assert_eq!(task.is_exec(&vars).unwrap(), true);
+    }
+
+    #[test]
+    fn test_is_exec_array_one_false() {
+        let s: String = r#"
+        when:
+          - false
+          - "boo == 'test'"
+        command: 'example'
+        "#
+        .to_owned();
+        let vars = vars::from_iter(vec![("boo", "test")].into_iter());
+        let out = YamlLoader::load_from_str(&s).unwrap();
+        let yaml = out.first().unwrap();
+        let task = Task::from(yaml);
+        assert_eq!(task.is_exec(&vars).unwrap(), false);
     }
 
     #[test]
@@ -441,13 +470,164 @@ mod tests {
           - 3
         "#
         .to_owned();
-        let vars = vars::from_iter(vec![("boo", "test")].into_iter());
+        let vars = vars::from_iter(vec![].into_iter());
         let out = YamlLoader::load_from_str(&s).unwrap();
         let yaml = out.first().unwrap();
         let task = Task::from(yaml);
         assert_eq!(task.render_iterator(vars).unwrap(), vec!["1", "2", "3"]);
     }
 
+    #[test]
+    fn test_is_changed() {
+        let s: String = r#"
+        changed_when: "boo == 'test'"
+        command: 'example'
+        "#
+        .to_owned();
+        let vars = vars::from_iter(vec![("boo", "test")].into_iter());
+        let out = YamlLoader::load_from_str(&s).unwrap();
+        let yaml = out.first().unwrap();
+        let task = Task::from(yaml);
+        assert_eq!(
+            task.is_changed(&ModuleResult::new(false, None, None), &vars)
+                .unwrap(),
+            true
+        );
+    }
+
+    #[test]
+    fn test_is_changed_bool_true() {
+        let s: String = r#"
+        changed_when: true
+        command: 'example'
+        "#
+        .to_owned();
+        let vars = vars::from_iter(vec![("boo", "test")].into_iter());
+        let out = YamlLoader::load_from_str(&s).unwrap();
+        let yaml = out.first().unwrap();
+        let task = Task::from(yaml);
+        assert_eq!(
+            task.is_changed(&ModuleResult::new(false, None, None), &vars)
+                .unwrap(),
+            true
+        );
+    }
+
+    #[test]
+    fn test_is_changed_bool_false() {
+        let s: String = r#"
+        changed_when: false
+        command: 'example'
+        "#
+        .to_owned();
+        let vars = vars::from_iter(vec![("boo", "test")].into_iter());
+        let out = YamlLoader::load_from_str(&s).unwrap();
+        let yaml = out.first().unwrap();
+        let task = Task::from(yaml);
+        assert_eq!(
+            task.is_changed(&ModuleResult::new(true, None, None), &vars)
+                .unwrap(),
+            false
+        );
+    }
+
+    #[test]
+    fn test_is_changed_string_false() {
+        let s: String = r#"
+        changed_when: "false"
+        command: 'example'
+        "#
+        .to_owned();
+        let vars = vars::from_iter(vec![("boo", "test")].into_iter());
+        let out = YamlLoader::load_from_str(&s).unwrap();
+        let yaml = out.first().unwrap();
+        let task = Task::from(yaml);
+        assert_eq!(
+            task.is_changed(&ModuleResult::new(false, None, None), &vars)
+                .unwrap(),
+            false
+        );
+    }
+
+    #[test]
+    fn test_is_changed_false() {
+        let s: String = r#"
+        changed_when: "boo != 'test'"
+        command: 'example'
+        "#
+        .to_owned();
+        let vars = vars::from_iter(vec![("boo", "test")].into_iter());
+        let out = YamlLoader::load_from_str(&s).unwrap();
+        let yaml = out.first().unwrap();
+        let task = Task::from(yaml);
+        assert_eq!(
+            task.is_changed(&ModuleResult::new(false, None, None), &vars)
+                .unwrap(),
+            false
+        );
+    }
+
+    #[test]
+    fn test_is_changed_array() {
+        let s: String = r#"
+        changed_when:
+          - "boo == 'test'"
+          - true
+        command: 'example'
+        "#
+        .to_owned();
+        let vars = vars::from_iter(vec![("boo", "test")].into_iter());
+        let out = YamlLoader::load_from_str(&s).unwrap();
+        let yaml = out.first().unwrap();
+        let task = Task::from(yaml);
+        assert_eq!(
+            task.is_changed(&ModuleResult::new(false, None, None), &vars)
+                .unwrap(),
+            true
+        );
+    }
+
+    #[test]
+    fn test_is_changed_array_false() {
+        let s: String = r#"
+        changed_when:
+          - "boo == 'test'"
+          - false
+        command: 'example'
+        "#
+        .to_owned();
+        let vars = vars::from_iter(vec![("boo", "test")].into_iter());
+        let out = YamlLoader::load_from_str(&s).unwrap();
+        let yaml = out.first().unwrap();
+        let task = Task::from(yaml);
+        assert_eq!(
+            task.is_changed(&ModuleResult::new(true, None, None), &vars)
+                .unwrap(),
+            false
+        );
+    }
+
+    #[test]
+    fn test_when_in_loop() {
+        let s: String = r#"
+        command: echo 'example'
+        loop:
+          - 1
+          - 2
+          - 3
+        when: item == 1
+        "#
+        .to_owned();
+        let vars = vars::from_iter(vec![].into_iter());
+        let out = YamlLoader::load_from_str(&s).unwrap();
+        let yaml = out.first().unwrap();
+        let task = Task::from(yaml);
+        let result = task.exec(vars).unwrap();
+        let mut expected = Context::new();
+        expected.insert("item", "3");
+        assert_eq!(result, expected);
+    }
+
     #[test]
     fn test_render_iterator_var() {
         let s: String = r#"
@@ -505,7 +685,7 @@ mod tests {
         "#
         )
         .unwrap();
-        let tasks = read_file(file_path).unwrap();
+        let tasks = read_file(file_path, false).unwrap();
         assert_eq!(tasks.len(), 2);
 
         let s0 = r#"
