@@ -4,21 +4,22 @@ use crate::vars::Vars;
 use tera::Tera;
 
 #[inline(always)]
-pub fn render_string(s: &str, vars: Vars) -> Result<String> {
+pub fn render_string(s: &str, vars: &Vars) -> Result<String> {
     let mut tera = Tera::default();
     trace!("rendering {:?}", &s);
-    tera.render_str(s, &vars)
+    tera.render_str(s, vars)
         .map_err(|e| Error::new(ErrorKind::InvalidData, e))
 }
 
 #[inline(always)]
-pub fn render_as_json(s: &str, vars: Vars) -> Result<String> {
+pub fn render_as_json(s: &str, vars: &Vars) -> Result<String> {
     render_string(&s.replace("}}", "| json_encode() | safe }}"), vars)
 }
 
 #[inline(always)]
-pub fn is_render_string(s: &str, vars: Vars) -> Result<bool> {
+pub fn is_render_string(s: &str, vars: &Vars) -> Result<bool> {
     match render_string(
+        // tera v2 will fix this allowing ({})
         &format!("{{% if {} | safe %}}true{{% else %}}false{{% endif %}}", s),
         vars,
     )?
@@ -28,3 +29,16 @@ pub fn is_render_string(s: &str, vars: Vars) -> Result<bool> {
         _ => Ok(true),
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_is_render_string() {
+        let r_true = is_render_string("true", &Vars::new()).unwrap();
+        assert_eq!(r_true, true);
+        let r_false = is_render_string("false", &Vars::new()).unwrap();
+        assert_eq!(r_false, false);
+    }
+}
